package com.soss.system.service.impl;

import com.soss.common.core.domain.entity.SysUser;
import com.soss.common.enums.CouponState;
import com.soss.common.exception.ServiceException;
import com.soss.common.utils.DateUtils;
import com.soss.system.domain.Coupon;
import com.soss.system.domain.CouponRule;
import com.soss.system.domain.CouponUser;
import com.soss.system.domain.vo.CouponVo;
import com.soss.system.mapper.CouponMapper;
import com.soss.system.mapper.CouponRuleMapper;
import com.soss.system.mapper.CouponUserMapper;
import com.soss.system.mapper.SysUserMapper;
import com.soss.system.service.ICouponUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 用户领取优惠券记录表 服务实现类
 * </p>
 *
 * @author caiyt
 * @since 2022-07-21
 */
@Service
public class CouponUserServiceImpl implements ICouponUserService {
    @Autowired
    private CouponUserMapper couponUserMapper;
    @Autowired
    private CouponMapper couponMapper;
    @Autowired
    private CouponRuleMapper couponRuleMapper;
    @Autowired
    private SysUserMapper userMapper;

    @Override
    public void giveUserCoupon(Long userId, String phone, Integer couponId) {
        SysUser user = userMapper.selectUserById(userId);
        Assert.isTrue(user != null && Objects.equals(user.getPhonenumber(), phone),
                "用户信息不匹配[id=" + userId + "][phone=" + phone + "]");
        Coupon coupon = couponMapper.selectCouponById(couponId);
        Assert.isTrue(coupon != null && Objects.equals(coupon.getState(), CouponState.ONLINE.getState()), "优惠券状态不合法");
        CouponRule couponRule = couponRuleMapper.selectCouponRuleById(coupon.getRuleId());
        Assert.notNull(couponRule, "未查询到匹配的优惠券规则[id=" + coupon.getRuleId() + "]");
        LocalDateTime now = LocalDateTime.now();
        if (couponRule.getUseStartTime().isAfter(now) || couponRule.getUseEndTime().isBefore(now)) {
            throw new ServiceException("优惠券不在有效期内");
        }
        CouponUser couponUser = new CouponUser();
        couponUser.setUserId(userId);
        couponUser.setUserName(user.getUserName());
        couponUser.setUserPhone(user.getPhonenumber());
        couponUser.setCouponId(couponId);
        couponUser.setExpiredTime(this.getCouponExpiredTime(couponRule, now));
        couponUser.setReceiveTime(now);
        couponUser.setSource("give");
        couponUser.setType(2);
        couponUser.setCreatedAt(now);
        couponUser.setUpdatedAt(now);
        couponUserMapper.insertCouponUser(couponUser);
    }

    @Override
    @Transactional
    public void receiveCoupon(Long userId, Integer id) {
        CouponUser couponUser = couponUserMapper.getUserCouponById(id);
        Assert.notNull(couponUser, "未查询到匹配记录[id=" + id + "]");
        Assert.isTrue(couponUser.getUserId().equals(userId), "该券与当前用户信息不符");
        LocalDateTime now = LocalDateTime.now();
        Coupon coupon = couponMapper.selectCouponById(couponUser.getCouponId());
        Assert.notNull(coupon, "未查询到匹配的优惠券信息[id=" + couponUser.getCouponId() + "]");
        CouponRule couponRule = couponRuleMapper.selectCouponRuleById(coupon.getRuleId());
        Assert.notNull(couponRule, "未查询到匹配的优惠券规则信息[id=" + coupon.getRuleId() + "]");
        if (couponRule.getReceivableTime() != null && couponRule.getReceivableTime().isAfter(now)) {
            throw new ServiceException("该券还未到可领取的时间：" + DateUtils.dateTimeNow());
        }
        if (couponRule.getUseEndTime().isBefore(now)) {
            throw new ServiceException("该券已失效");
        }
        couponUser.setExpiredTime(this.getCouponExpiredTime(couponRule, now));
        couponUser.setReceiveTime(now);
        couponUser.setUpdatedAt(now);
        couponUserMapper.updateCouponUser(couponUser);
    }

    private LocalDateTime getCouponExpiredTime(CouponRule couponRule, LocalDateTime now) {
        LocalDateTime expiredTime;
        if (couponRule.getRelativeTime() != null && couponRule.getRelativeTime() > 0) {
            LocalDateTime expireComputerStartTime = couponRule.getUseStartTime().isAfter(now) ? couponRule.getUseStartTime() : now;
            expiredTime = DateUtils.addDaysAndGetEnd(expireComputerStartTime, couponRule.getRelativeTime());
            expiredTime = expiredTime.isBefore(couponRule.getUseEndTime()) ? expiredTime : couponRule.getUseEndTime();
        } else {
            expiredTime = couponRule.getUseEndTime();
        }
        return expiredTime;
    }

    /**
     * 查询用户可用优惠券数量
     *
     * @param userId
     * @return
     */
    @Override
    public int getUserAvailableCouponCnt(Long userId) {
        return couponUserMapper.getUserAvailableCouponCnt(userId, LocalDateTime.now());
    }

    /**
     * 查询用户可用优惠券列表
     *
     * @param userId
     * @return
     */
    @Override
    public List<CouponVo> listUserAvailableCoupon(Long userId) {
        return couponUserMapper.listUserAvailableCoupon(userId, LocalDateTime.now());
    }
}
